<?php
/** miniOrange enables user to log in using LDAP credentials.
    Copyright (C) 2015  miniOrange

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
* @package 		miniOrange OAuth
* @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/
/**
This library is miniOrange Authentication Service. 
Contains Request Calls to LDAP Service.

**/


class Mo_Ldap_Role_Mapping{
	
	
	function __construct() {
		
	}
	
	
	function get_member_of_attribute($username) {
		$membersarray = array();
		
		$url = get_option('mo_ldap_host_name')  . "/moas/api/ldap/getmemberof";
		//calls to encrypt username and password
		
		$customer_id = get_option('mo_ldap_admin_customer_key') ? get_option('mo_ldap_admin_customer_key') : null;
		$memberOf = get_option('mo_ldap_mapping_memberof_attribute');
		if(!$memberOf)
			$memberOf = "memberof";
		$data = array(
				'customerId' => $customer_id,
				'userName' => $username,
				"memberOf" => $memberOf,
				'ldapAuditRequest' => array(
						'endUserEmail' => $username,
						'applicationName' => $_SERVER['SERVER_NAME'],
						'appType' => 'WP LDAP for Cloud',
						'requestType' => "Get Member Of Attribute"
				)
		);
		$data_string = json_encode($data);
		
		//Authorization Header
		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_ldap_admin_customer_key' );
		
		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_ldap_admin_api_key' );
		
		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );
		
		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );
		
		$customerKeyHeader = "Customer-Key: " . $customerKey;
		$timestampHeader = "Timestamp: " . number_format ( $currentTimeInMillis, 0, '', '' );
		$authorizationHeader = "Authorization: " . $hashValue;

		$curl = curl_init();
		curl_setopt_array($curl, array(
		CURLOPT_RETURNTRANSFER => 1,
		CURLOPT_URL => $url,
		CURLOPT_POST => 1,
		CURLOPT_POSTFIELDS => $data_string,
		CURLOPT_FOLLOWLOCATION => 1,
		CURLOPT_SSL_VERIFYPEER => false,
		CURLOPT_HTTPHEADER => array(
		'Content-Type: application/json',
		'Content-Length: ' . strlen($data_string), 
		$customerKeyHeader,
		$timestampHeader,
		$authorizationHeader,
		)
		));
		$response = curl_exec($curl);
		$decoded_response = (array)json_decode($response);
		if(isset($decoded_response['groups']))
		 	return $decoded_response['groups'];
		return array(); 
	}
	
	
	function mo_ldap_update_role_mapping($user_id,$member_of_attr_array) {
		if($user_id==1) {
			return;
		}
			
		$roles = 0; //initialize role counter
		$mapping_count=0;
		if(is_numeric(get_option('mo_ldap_role_mapping_count'))) {
			$mapping_count = intval(get_option('mo_ldap_role_mapping_count'));
		}
	    $keep_existing_roles = get_option('mo_ldap_keep_existing_user_roles');
        error_log("Keep existing user roles value : ". $keep_existing_roles);
        
		$wpuser = new WP_User($user_id);
		for($i = 1 ; $i <= $mapping_count ; $i++){
			$group = get_option( 'mo_ldap_mapping_key_'.$i);
			$matches = preg_grep("/".$group."/i", $member_of_attr_array);
			if(count($matches)>0){
				$groupMapping = get_option( 'mo_ldap_mapping_value_'.$i);
				if($groupMapping)
				{
				 error_log("adding role ".$groupMapping);
                  if ($keep_existing_roles == 1) {
                      $wpuser->add_role($groupMapping);
                    } else {
					if($roles==0)
						$wpuser->set_role('');
					$wpuser->add_role( $groupMapping );
				}
					$roles++;
				}
					
			}
		}
			
		//If no match found set role to default
		if ($roles == 0) {
            error_log("Setting default role ".get_option('mo_ldap_mapping_value_default'));
		if(get_option('mo_ldap_mapping_value_default')) {
				if ($keep_existing_roles == 1) {
				$wpuser->add_role( get_option('mo_ldap_mapping_value_default') );
				} else {
                    $wpuser->set_role(get_option('mo_ldap_mapping_value_default'));
                }
			}
		}
	}


	function test_configuration($username){
		if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
			return null;
		}
		
		$groups = $this->get_member_of_attribute($username);
		
		$mapping_count=0;
		if(is_numeric(get_option('mo_ldap_role_mapping_count'))) {
			$mapping_count = intval(get_option('mo_ldap_role_mapping_count'));
		}
		echo "<div style=padding:20px>";
		$flag=0;
		$assigned_roles = array();
		for($i = 1 ; $i <= $mapping_count ; $i++){
			$group = get_option( 'mo_ldap_mapping_key_'.$i);
			$matches = preg_grep("/".$group."/i", $groups);
			if(count($matches)>0){
				$groupMapping = get_option( 'mo_ldap_mapping_value_'.$i);
				if($groupMapping)
				{
					if($flag==0)
						echo '<div style="color: #3c763d;background-color: #dff0d8; padding:2%;margin-bottom:20px;text-align:center; border:1px solid #AEDB9A; font-size:18pt;">TEST SUCCESSFUL</div>
							<div style="display:block;text-align:center;margin-bottom:4%;"><img style="width:15%;"src="' . plugin_dir_url(__FILE__) . 'includes/images/green_check.png' .'"/></div>';
                    echo "<li>User <b>".$username."</b> found in group <b>".$group."</b> which matches role <b>".$groupMapping.".</b></li>";
                    $flag+=1;
                    array_push($assigned_roles,$groupMapping);

				}
					
			}
		}
		if($flag==0){
			echo '<div style="color: #a94442;background-color: #f2dede;padding: 15px;margin-bottom: 20px;text-align:center;border:1px solid #E6B3B2;font-size:18pt;">TEST FAILED</div><div style="display:block;text-align:center;margin-bottom:4%;"><img style="width:15%;"src="'. plugin_dir_url(__FILE__) . 'includes/images/wrong.png' . '"/></div>
						<span>User <b>'.$username.'</b> not found in any group specified in role mapping.</span><br>';
						if(get_option('mo_ldap_mapping_value_default')) echo " Default Role <b>".get_option('mo_ldap_mapping_value_default')."</b> will be assigned to the User. ";
						if(get_option('mo_ldap_free_version', '1') != 0){
								echo "<br><br>Please check : 
								<li>If you have specified DN for group name e.g.<b>cn=group,dc=domain,dc=com</b></li>
								<li>If you have added users in groups specified in role mapping.</li>";
						}
		}else if($flag==1){
					echo "<br><br><li>Role <b>".$groupMapping."</b> will be assigned to the User.</li>";
		} else{
				$assigned_roles=array_unique($assigned_roles);
            echo "</br><li>Following roles will be assigned to the User:</li>";
            echo "<ol>";
            foreach ($assigned_roles as $role) {
                echo "<li><b>" . $role . "</b></li>";
            }
        }
		echo "</div>";
		
		
		exit();
	}
		
}

?>